function varargout = MC_ParticleSim(varargin)
% MC_CALCPI Application M-file for MC_ParticleSim.fig
%    FIG = MC_CALCPI launch MC_ParticleSim GUI.
%    MC_CALCPI('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.0 22-Jan-2003 11:26:03

if nargin == 0  % LAUNCH GUI

	fig = openfig(mfilename,'reuse');

	% Generate a structure of handles to pass to callbacks, and store it. 
	handles = guihandles(fig);
	guidata(fig, handles);

   
	if nargout > 0
		varargout{1} = fig;
	end
    
    % set value for iterations and nr of particles
    set( handles.Iterations_Slider, 'Value', 100 );
    set( handles.Iterations_Edit,'String', num2str(get(handles.Iterations_Slider,'Value')));
   
    %
    % load logos
    %
    ITPLogo = imread('logo-itp.png');
    image( ITPLogo, 'Parent', handles.ITPLogo );
    set( handles.ITPLogo, 'Visible', 'off');            % disable axes
    
    TUGLogo = imread('logo-tug.png');
    image( TUGLogo, 'Parent', handles.TUGLogo );
    set( handles.TUGLogo, 'Visible', 'off');            % disable axes
    
    
elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

	try
		[varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
	catch
		disp(lasterr);
	end

end


%| ABOUT CALLBACKS:
%| GUIDE automatically appends subfunction prototypes to this file, and 
%| sets objects' callback properties to call them through the FEVAL 
%| switchyard above. This comment describes that mechanism.
%|
%| Each callback subfunction declaration has the following form:
%| <SUBFUNCTION_NAME>(H, EVENTDATA, HANDLES, VARARGIN)
%|
%| The subfunction name is composed using the object's Tag and the 
%| callback type separated by '_', e.g. 'slider2_Callback',
%| 'figure1_CloseRequestFcn', 'axis1_ButtondownFcn'.
%|
%| H is the callback object's handle (obtained using GCBO).
%|
%| EVENTDATA is empty, but reserved for future use.
%|
%| HANDLES is a structure containing handles of components in GUI using
%| tags as fieldnames, e.g. handles.figure1, handles.slider2. This
%| structure is created at GUI startup using GUIHANDLES and stored in
%| the figure's application data using GUIDATA. A copy of the structure
%| is passed to each callback.  You can store additional information in
%| this structure at GUI startup, and you can change the structure
%| during callbacks.  Call guidata(h, handles) after changing your
%| copy to replace the stored original so that subsequent callbacks see
%| the updates. Type "help guihandles" and "help guidata" for more
%| information.
%|
%| VARARGIN contains any extra arguments you have passed to the
%| callback. Specify the extra arguments by editing the callback
%| property in the inspector. By default, GUIDE sets the property to:
%| <MFILENAME>('<SUBFUNCTION_NAME>', gcbo, [], guidata(gcbo))
%| Add any extra arguments after the last argument, before the final
%| closing parenthesis.

% --------------------------------------------------------------------

% --------------------------------------------------------------------

function varargout = Iterations_Edit_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
% Stub for Callback of the uicontrol handles.Iterations_Edit.
val = str2num(get(handles.Iterations_Edit,'String'));
if isnumeric(val) & length(val)==1 & ...
    val >= get(handles.Iterations_Slider,'Min') & ...
    val <= get(handles.Iterations_Slider,'Max')
    set(handles.Iterations_Slider,'Value',val);
else
% Increment the error count, and display it
    handles.numberOfErrors = handles.numberOfErrors+1;
    set(handles.Iterations_Edit,'String', [handles.errorString,num2str(handles.numberOfErrors)]);
    guidata(gcbo,handles); % store the changes...
end


% --------------------------------------------------------------------

function varargout = Iterations_Slider_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
% Stub for Callback of the uicontrol handles.Iterations_Slider.
set( handles.Iterations_Edit,'String', num2str(get(handles.Iterations_Slider,'Value')));
guidata(gcbo,handles);



% --------------------------------------------------------------------

function varargout = RunBtn_Callback(h, eventdata, handles, varargin)

% set status message
set( handles.StatusText, 'String', 'Status: running....' );

% --------------------------------------------------------------------
% Stub for Callback of the uicontrol handles.RunBtn.
NrOfIterations = get( handles.Iterations_Slider, 'Value');

% clear figures
CircleScale = 20;
HitImage = zeros( CircleScale, CircleScale );

%
% Using Monte Carlo integration to calculate pi
%
% Shooting randomly at a square that represents the bounding
% box of a circle, leaves us with a certain probability to 
% end up with a hit inside the circle. This probability equals
% the relation: Area_Circle / AreaSquare = Hits / Shots.
% Using the relation r^2*Pi = Area_Circle and AreaSquare = 4*r^2
% we can easily calculate Pi = 4 * Hits / Shots
%
% Author:  Chris Theis          ctheis@itp.tu-graz.ac.at
% Version: 1.0					03/12/01
%

FrameRate = floor( NrOfIterations / 20 );
Hits = 0;
CalculatedPi = zeros( NrOfIterations, 1);
Error = zeros( NrOfIterations, 1);

for Shots=1:NrOfIterations
   x = -1 + 2*rand(1);		% produce random numbers between -1 .. 1
   y = -1 + 2*rand(1);
   
   IndexX = 1 + floor( (x + 1) * CircleScale / 2);
   IndexY = 1 + floor( (y + 1) * CircleScale / 2);
   
   % check if we are inside our circle
   if( x^2 + y^2 < 1 )
      Hits = Hits + 1;
      if( HitImage( IndexX, IndexY ) ~= 15 )    
          HitImage( IndexX, IndexY ) = 35;              % color hit within circle
      end
   else
      if( HitImage( IndexX, IndexY ) ~= 35 )
          HitImage( IndexX, IndexY ) = 15;              % color hit outside circle
      end
   end

   CalculatedPi(Shots) = 4 * Hits / Shots;
   Error(Shots ) = abs( pi - CalculatedPi(Shots) );
   
end

% plot hits
image( HitImage, 'Parent', handles.CircleFigure )    

% plot exact values
X = 1:NrOfIterations;
Y = 4 * atan(1.0);
line( X, Y, 'Parent', handles.PiFigure )

% plot calculated pi
plot( CalculatedPi, 'Parent', handles.PiFigure )

% plot error
plot( X, Error, 'Parent', handles.ErrorFig )

% output values
MeanPi = sum( CalculatedPi ) / NrOfIterations;
MeanRMS = sqrt( sum( (CalculatedPi - MeanPi).^2) / ( NrOfIterations) );

set( handles.StatusText, 'String', 'Status: stopped....' );
set( handles.PiValueText, 'String', sprintf('Calculated value of Pi: %f +- %f', MeanPi, MeanRMS ))
end

   
      
