function varargout = MC_ParticleSim(varargin)
% MC_MEANDIST Application M-file for MC_ParticleSim.fig
%    FIG = MC_MEANDIST launch MC_ParticleSim GUI.
%    MC_MEANDIST('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.0 22-Jan-2003 11:26:03

if nargin == 0  % LAUNCH GUI

	fig = openfig(mfilename,'reuse');

	% Generate a structure of handles to pass to callbacks, and store it. 
	handles = guihandles(fig);
	guidata(fig, handles);

   
	if nargout > 0
		varargout{1} = fig;
	end
    
    % set value for iterations and nr of particles
    set( handles.Iterations_Slider, 'Value', 1000 );
    set( handles.GridSize_Slider, 'Value', 25 );    
    set( handles.Radius_Slider, 'Value', 1 );        
    set( handles.Iterations_Edit,'String', num2str(get(handles.Iterations_Slider,'Value')));
    set( handles.GridSize_Edit,'String', num2str(get(handles.GridSize_Slider,'Value')));
    set( handles.Radius_Edit,'String', num2str(get(handles.Radius_Slider,'Value')));    
    
    %
    % load logos
    %
    ITPLogo = imread('logo-itp.png');
    image( ITPLogo, 'Parent', handles.ITPLogo );
    set( handles.ITPLogo, 'Visible', 'off');            % disable axes
    
    TUGLogo = imread('logo-tug.png');
    image( TUGLogo, 'Parent', handles.TUGLogo );
    set( handles.TUGLogo, 'Visible', 'off');            % disable axes
    
    
elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

	try
		[varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
	catch
		disp(lasterr);
	end

end


%| ABOUT CALLBACKS:
%| GUIDE automatically appends subfunction prototypes to this file, and 
%| sets objects' callback properties to call them through the FEVAL 
%| switchyard above. This comment describes that mechanism.
%|
%| Each callback subfunction declaration has the following form:
%| <SUBFUNCTION_NAME>(H, EVENTDATA, HANDLES, VARARGIN)
%|
%| The subfunction name is composed using the object's Tag and the 
%| callback type separated by '_', e.g. 'slider2_Callback',
%| 'figure1_CloseRequestFcn', 'axis1_ButtondownFcn'.
%|
%| H is the callback object's handle (obtained using GCBO).
%|
%| EVENTDATA is empty, but reserved for future use.
%|
%| HANDLES is a structure containing handles of components in GUI using
%| tags as fieldnames, e.g. handles.figure1, handles.slider2. This
%| structure is created at GUI startup using GUIHANDLES and stored in
%| the figure's application data using GUIDATA. A copy of the structure
%| is passed to each callback.  You can store additional information in
%| this structure at GUI startup, and you can change the structure
%| during callbacks.  Call guidata(h, handles) after changing your
%| copy to replace the stored original so that subsequent callbacks see
%| the updates. Type "help guihandles" and "help guidata" for more
%| information.
%|
%| VARARGIN contains any extra arguments you have passed to the
%| callback. Specify the extra arguments by editing the callback
%| property in the inspector. By default, GUIDE sets the property to:
%| <MFILENAME>('<SUBFUNCTION_NAME>', gcbo, [], guidata(gcbo))
%| Add any extra arguments after the last argument, before the final
%| closing parenthesis.

% --------------------------------------------------------------------
function varargout = Radius_Edit_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.InvTemp_Edit.
val = str2num(get(handles.Radius_Edit,'String'));
if isnumeric(val) & length(val)==1 & ...
    val >= get(handles.Radius_Slider,'Min') & ...
    val <= get(handles.Radius_Slider,'Max')
    set(handles.Radius_Slider,'Value',val);
else
% Increment the error count, and display it
    handles.numberOfErrors = handles.numberOfErrors+1;
    set(handles.Radius_Edit,'String', [handles.errorString,num2str(handles.numberOfErrors)]);
    guidata(gcbo,handles); % store the changes...
end


% --------------------------------------------------------------------
function varargout = Radius_Slider_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.InvTemp_Slider.
set( handles.Radius_Edit,'String', num2str(get(handles.Radius_Slider,'Value')));
guidata(gcbo,handles);


% --------------------------------------------------------------------
function varargout = GridSize_Slider_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.NrOfParticles_Slider.
set( handles.GridSize_Edit,'String', num2str(get(handles.GridSize_Slider,'Value')));
guidata(gcbo,handles);


% --------------------------------------------------------------------
function varargout = GridSize_Edit_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.NrOfParticles_Edit.
val = str2num(get(handles.GridSize_Edit,'String'));
if isnumeric(val) & length(val)==1 & ...
    val >= get(handles.GridSize_Slider,'Min') & ...
    val <= get(handles.GridSize_Slider,'Max')
    set(handles.GridSize_Slider,'Value',val);
else
% Increment the error count, and display it
    handles.numberOfErrors = handles.numberOfErrors+1;
    set(handles.GridSize_Edit,'String', [handles.errorString,num2str(handles.numberOfErrors)]);
    guidata(gcbo,handles); % store the changes...
end

% --------------------------------------------------------------------
function varargout = Iterations_Edit_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.Iterations_Edit.
val = str2num(get(handles.Iterations_Edit,'String'));
if isnumeric(val) & length(val)==1 & ...
    val >= get(handles.Iterations_Slider,'Min') & ...
    val <= get(handles.Iterations_Slider,'Max')
    set(handles.Iterations_Slider,'Value',val);
else
% Increment the error count, and display it
    handles.numberOfErrors = handles.numberOfErrors+1;
    set(handles.Iterations_Edit,'String', [handles.errorString,num2str(handles.numberOfErrors)]);
    guidata(gcbo,handles); % store the changes...
end


% --------------------------------------------------------------------
function varargout = Iterations_Slider_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.Iterations_Slider.
set( handles.Iterations_Edit,'String', num2str(get(handles.Iterations_Slider,'Value')));
guidata(gcbo,handles);


% --------------------------------------------------------------------
%
% Minimization of the mean distance for an ensemble of particles with
% a hard core potential as an additional constraint.
%
% This program demonstrates the evaluation of a high dimensional integral
% in 2n phase space with simple constraints. For a detailed description
% please refer to the provided PDF file.
%
%
% Author:  Chris Theis          ctheis@itp.tu-graz.ac.at
% Version: 1.0					03/12/01
%
% --------------------------------------------------------------------
function varargout = RunBtn_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.RunBtn.
GridSize = get( handles.GridSize_Slider, 'Value');
NrOfIterations = get( handles.Iterations_Slider, 'Value');
ParticleRadius = get( handles.Radius_Slider, 'Value');

FrameRate  = floor(NrOfIterations / 10);
Population = 10;                            % populate 30 percent of the grid
ParticleColor = 35;

% set up initial condition
r = rand( GridSize, GridSize );

while( NrOfIterations < FrameRate )
    disp('Impossible values entered!');
    NrOfIterations = input('Nr. of timesteps: ');
    FrameRate  = input('Update every #th frame: ');
end

Space = zeros( GridSize, GridSize );
MeanDist = zeros( NrOfIterations, 1);
bFlag = 0;

% populate phase space
r = rand( GridSize, GridSize );
Index = find(r < (Population / 100) );        
NrOfParticles = length(Index);
Space( Index ) = ParticleColor;

% let`s go

Index = find(Space == ParticleColor);
for t=1:NrOfIterations
    
    while( bFlag == 0)

        Temp = 0;        
        % randomly choose a particle
        ParticleNumber = floor( rand(1) * length(Index)) + 1;
        Pos = Index( ParticleNumber );
        x = floor( rand(1) * GridSize ) + 1;
        y = floor( rand(1) * GridSize ) + 1;
    
        % move particle
        Space( Pos ) = 0;
        Space( x, y) = ParticleColor;
    
        % check for rejection condition
        ParticleX = floor( Pos / GridSize ) + 1;        % calculate X, Y coordinate
        ParticleY = mod( Pos, GridSize );               % within the matrix from the linear index
    
        for i=1:(ParticleNumber - 1)
            X = floor( Index( i ) / GridSize ) + 1;
            Y = mod( Index( i ), GridSize );
            
            Distance = sqrt( (X - ParticleX)^2 + (Y - ParticleY)^2);
            Temp = Temp + Distance;
            if( Distance < ParticleRadius )
                Space( x, y) = 0;               % reset particle position
                Space( Pos ) = ParticleColor;
                bFlag = 0;
                break;
            else
                bFlag = 1;
            end
        end

        for i=(ParticleNumber + 1):length(Index)
            if( bFlag == 0)
                break;
            end
               
            X = floor( Index( i ) / GridSize ) + 1;
            Y = mod( Index( i ), GridSize );

            Distance = sqrt( (X - ParticleX)^2 + (Y - ParticleY)^2);
            Temp = Temp + Distance;
            if( Distance < ParticleRadius )
                Space( x, y) = 0;               % reset particle position
                Space( Pos ) = ParticleColor;
                bFlag = 0;
                break;
            else
                bFlag = 1;
            end
            
        end
    end
    bFlag = 0;
    
    % update position index of our particles
    Index = find(Space == ParticleColor);
    
    % calculate current mean distance
    MeanDist(t) = Temp / NrOfParticles;

    if( mod( t, FrameRate ) == 0 | t == 1)
        image( Space, 'Parent', handles.ParticlePosFigure )
        plot( MeanDist, 'Parent', handles.MeanDistFigure )
        drawnow
    end
    
end

%histogram( MeanDist, NrOfIterations, 0, 20, 20);

disp( sprintf( 'Mean distance over the whole phase space: %f nm', sum( MeanDist ) / NrOfIterations ) )
line( 1:NrOfIterations, sum( MeanDist ) / NrOfIterations, 'Parent', handles.MeanDistFigure )


