% --------------------------------------------------------------------
%
% Calculate a possible evolution of stock prices using a (very)
% simplified model based on a diffusion process (Brownian motion).
%
% This program is intended as a simple example for the application of
% Monte Carlo methods in finance. A more realistic implementation and
% analysis would require a deeper knowledge of financial theory.
% As this is intended as an outlook for possible applications and not
% as an application for stock brokers or other professionals, please
% refer to the bibliography section of the PDF file for further information.
%
% Author:   Chris Theis         03/21/02
%           ctheis@itp.tu-graz.ac.at
%
% --------------------------------------------------------------------

function RetPrice = MC_Stock( Iterations, Mu, Sigma )

MAX_STEP = 365;      % Simulate a stock price for one year
DT = 1.0/ MAX_STEP;  % Time step (a day ) in unit of a year 
%Mu = 0.14;           % growth rate per annum
%Sigma = 0.20;        % volatility
NrOfWalkers = Iterations;  % Nr of random walkers
InitialPrice = 20.0;
NrOfBins = 50;

Mu_dt = Mu * DT;                        % abbreviations for terms of the stochastic equation
Sigma_rtDT = Sigma * sqrt(DT);          % abbreviations for terms of the stochastic equation
Price = zeros(MAX_STEP, 1);

for Walker = 0:NrOfWalkers
    Price(1) = InitialPrice;
    for Day=2:MAX_STEP
        
        % calculate diffusion process for the evolution of the stock price
        Diffusion = normrnd(0,1);
        Price(Day) = Price(Day-1) * exp(Mu_dt + Sigma_rtDT * Diffusion);
        
    end
    
end

RetPrice = Price;
