%
% Calculate the volume of a n-dimensional sphere using Monte Carlo integration
% (c) 03/18/02 Chris Theis
%
%

clear all;
MaxDim = 20;
TimeVector = zeros(MaxDim,1);
ErrorMC = zeros( MaxDim, 1);
ErrorTrapez = zeros( MaxDim, 1);

for Dim = 2:MaxDim

  NrOfIterations = 50000;
  Counter = 0;
  RadiusSquared = 1.0;

  Vector = zeros( Dim, 1 );
  tic;
  for i = 1:NrOfIterations
     Vector = rand( Dim, 1);
     if( sum(Vector .* Vector) < RadiusSquared )
        Counter = Counter + 1;
     end
  end
  TimeVector( Dim, 1) = toc;
  Volume = 2^Dim * Counter / NrOfIterations;
  ErrorMC( Dim, 1 ) = 1./ sqrt(NrOfIterations);
end

%
% calculate the integral using the ordinary trapez formula

VolOld = 2;                     % volume of the 1-d sphere
TimeVector2 = zeros(MaxDim,1);

for CurDim = 2:MaxDim
    tic;
    x = -1:(1/NrOfIterations):1;
    VolOld = 2;                     % volume of the 1-d sphere
    for Dim = 1:CurDim              % integrate over all dimensions
      Y = (1-x.^2).^(Dim/2);
      Volume = VolOld * trapz(x,Y);
      VolOld = Volume;
    end
%    Volume
    TimeVector2( CurDim, 1) = toc;
    ErrorTrapez( CurDim, 1 ) = 1 / (NrOfIterations^(2/CurDim));
end

clf;
figure(1);
plot( 1:MaxDim, TimeVector, 'r+', 1:MaxDim, TimeVector2, 'bo')
xlabel('Dimension/1')
ylabel('Time/s')

figure(2);
plot( 1:MaxDim, ErrorMC, 'r', 1:MaxDim, ErrorTrapez, 'b' );
xlabel('Dimension/1');
ylabel('Error');